from __future__ import annotations
from leadmo_api.models.common import BaseLeadmoApiCallModel


class GetWorkflowsParams(BaseLeadmoApiCallModel):
    """
    Parameters for retrieving all workflows associated with the user.
    """

    pass
